%% Compare with PageRank
% The idea in this experiment is to take the output from
% driver_compare_with_pagerank and actually compute the ktau coefficients.
% To run this experiment, we must enable a larger Java memory.  

%% Setup the experiment
% This experiment should be run from the rapr/experiments/comparison directory
cwd = pwd;
dirtail = 'experiments/comparison'; 
if strcmp(cwd(end-length(dirtail)+1:end),dirtail) == 0
    warning('rapr:dir','%s should be executed from rapr/%s\n',mfilename,dirtail);
end
addpath('../../matlab');
addpath('../tools');

%%
% Type a copy of ourselves so we have a record
type driver_compare_with_pagerank.m

%% 
% Load the results
load 'compare-results-final.mat'

%% Compute results statistics
ttol = 1e-8;
rstats=[];
for i=1:length(results)
    xeA=results(i).xeA; ex=results(i).ex; stdx=results(i).stdx;
    tic
    rstats(i).diff.ex=norm(xeA-ex,1);
    rstats(i).diff.stdx=norm(xeA-stdx,1);
    rstats(i).tau.ex=ktau(xeA,ex);
    rstats(i).tau.stdx=ktau(xeA,stdx);
    txeA=round(xeA/ttol)*ttol; tex=round(ex/ttol)*ttol; tstdx=round(stdx/ttol)*ttol;
    rstats(i).ttau.ex=ktau(txeA,tex);
    rstats(i).ttau.stdx=ktau(txeA,tstdx);
    rstats(i).dt=toc;
    fprintf('... finished %s with %s in %f sec ...\n', ...
        results(i).graph, results(i).d.name, toc);
    save 'compare-results-stats.mat' rstats
end
save 'compare-results-stats.mat' rstats

%%
% Reformat results for easier manipulation inside Matlab
rmat = zeros(length(rstats),6); rgraph=cell(length(rstats),1); rdist=rgraph;
for i=1:length(rstats)
    rmat(i,1)=rstats(i).diff.ex; rmat(i,4)=rstats(i).diff.stdx;
    rmat(i,2)=rstats(i).tau.ex; rmat(i,5)=rstats(i).tau.stdx;
    rmat(i,3)=rstats(i).ttau.ex; rmat(i,6)=rstats(i).ttau.stdx;
    rgraph{i}=results(i).graph; rdist{i}=results(i).d.name;
end

%% Format output
% I haven't written this section yet, that'll come after we get some
% results to play with :-)

% define a function to print a cell color
ccv = @(x) [1 2-x 2-x].*[1 0.5 0.5]*(x>=0)+... %  red for pos
    [x+2 x+2 1].*[0.5 0.5 1]*(x<0);            % blue for neg
ccc = @(x) sprintf('\\cellcolor[rgb]{%4.2f,%4.2f,%4.2f}',ccv(x));


fprintf('Comparison between x(E(A)),E(x(A)),and Std(x(A))\n');

fprintf('... latex table code ... \n'); 
diary off;
!rm pagerank-comparison-data.tex
diary pagerank-comparison-data.tex
gn=''; % graph name
for i=1:length(rstats)
    if ~strcmp(rgraph{i},gn)
        gn=rgraph{i}; 
        fprintf('\\midrule \n');
        fprintf('\\dataname{%20s} & ',gn);
    else fprintf('%31s & ',''); end
    fprintf('%20s & ', strrep(rdist{i}(6:end-1),',','&'));
    fprintf('\n %4s','& '); % print indent
    v=1-rmat(i,1); fprintf('%31s %5.3f & ',   ccc(v), v); 
    v=  rmat(i,2); fprintf('%31s %5.3f & ',   ccc(v), v); 
    v=  rmat(i,3); fprintf('%31s %5.3f & ',   ccc(v), v); 
    fprintf('\n %4s','& '); % print indent
    v=1-rmat(i,4); fprintf('%31s %5.3f & ',   ccc(v), v); 
    v=  rmat(i,5); fprintf('%31s %5.3f & ',   ccc(v), v); 
    v=  rmat(i,6); fprintf('%31s %5.3f   ',   ccc(v), v); 
    fprintf('\\\\ \n');
end
diary off;