%% Analyze the results from computations on the uk2006 graph
% The uk2006 graph is a large ...
% This code will be documented and explained soon...

%% Setup the experiment
% This experiment should be run from the rapr/experiments/comparison directory
cwd = pwd;
dirtail = 'experiments/comparison'; 
if strcmp(cwd(end-length(dirtail)+1:end),dirtail) == 0
    warning('rapr:dir','%s should be executed from rapr/%s\n',mfilename,dirtail);
end
addpath('../tools');
addpath('../../matlab');

%% 
% Load the results
clear;
results = [];
results(end+1).x = binload('../webspam/uk-2006-05.50.pr');
results(end).name = 'pagerank 0.5';
results(end).texname = '\vx(0.5)';
results(end+1).x = binload('../webspam/uk-2006-05.85.pr');
results(end).name = 'pagerank 0.85';
results(end).texname = '\vx(0.85)';
results(end+1).x = binload('../webspam/uk-2006-05.95.pr');
results(end).name = 'pagerank 0.95';
results(end).texname = '\vx(0.95)';
results(end+1).x = binload('../uk-spam-2006/uk-2006-05-beta-2-16-21.mean');
results(end).name = 'ex beta(2,16)';
results(end).texname = '\Eof{\vx(A_1)}';
results(end+1).x = binload('../uk-spam-2006/uk-2006-05-beta-1-1-10.mean');
results(end).name = 'ex beta(1,1)';
results(end).texname = '\Eof{\vx(A_2)}';
results(end+1).x = binload('../uk-spam-2006/uk-2006-05-beta-2-16-21.std');
results(end).name = 'stdx beta(2,16)';
results(end).texname = '\Stdof{\vx(A_1)}';
results(end+1).x = binload('../uk-spam-2006/uk-2006-05-beta-1-1-10.std');
results(end).name = 'stdx beta(1,1)';
results(end).texname = '\Stdof{\vx(A_2)}';

%%
nx = length(results);
ndiff = zeros(nx,nx);
ntau = zeros(nx,nx);
nttau = zeros(nx,nx);
ttol = 1e-10;


%%
% Compute all the pairwise comparisons between these vectors
tic
for i=1:nx
    for j=i+1:nx
        x = results(i).x; y = results(j).x;
        tx = round(x/ttol)*ttol; ty = round(y/ttol)*ttol;
        ndiff(i,j) = norm(x-y,1);
        ntau(i,j) = ktau(x,y);
        nttau(i,j) = ktau(tx,ty);
        save 'uk-2006-results.mat' ndiff ntau nttau;
    end
end
toc

%%
% Load and plot the output.  For the saved scripts, this code wasn't
% complete.

% define a function to print a cell color
ccv = @(x) [1 2-x 2-x].*[1 0.5 0.5]*(x>=0)+... %  red for pos
    [x+2 x+2 1].*[0.5 0.5 1]*(x<0);            % blue for neg
ccc = @(x) sprintf('\\cellcolor[rgb]{%4.2f,%4.2f,%4.2f}',ccv(x));

fprintf('... latex table code ... \n'); 

% display only the ktau values between deterministic PageRank and all
% and exPageRank and all, that is, don't display the std to std results
!rm uk-2006-data.tex
diary uk-2006-data.tex
exs=[1 2 3 4 5];
fprintf('\n\n\n Tau table \n\n\n');
for i=exs, 
    fprintf('$%15s$ & \n', results(i).texname);
    for j=2:nx
        if j<=i, fprintf('%4s & \n', ''); continue; end
        val=ntau(i,j); fprintf('%4s %31s %6.3f ', '', ccc(val),val);
        if j<nx, fprintf(' & \n'); else fprintf('   \n'); end
    end
    fprintf('\\\\ \n');
end

fprintf('\n\n\n Truncated tau table \n\n\n');

exs=[1 2 3 4 5];
for i=exs, 
    fprintf('$%15s$ & \n', results(i).texname);
    for j=2:nx
        if j<=i, fprintf('%4s & \n', ''); continue; end
        val=nttau(i,j); fprintf('%4s %31s %6.3f ', '', ccc(val),val);
        if j<nx, fprintf(' & \n'); else fprintf('   \n'); end
    end
    fprintf('\\\\ \n');
end
diary off;

%% Compute top-k intersection metric
% This section computes the top k intersection metric for the standard
% deviation and expectation vectors.  This m

% reset and load the data
clear;

temp = binload('../webspam/uk-2006-05.85.pr'); 
[sxeA1 pxeA1]=sort(temp,1,'descend');
temp = binload('../webspam/uk-2006-05.50.pr');
[sxeA2 pxeA2]=sort(temp,1,'descend');
temp = binload('../uk-spam-2006/uk-2006-05-beta-2-16-21.std');
[sstdx1 pstdx1]=sort(temp,1,'descend');
temp = binload('../uk-spam-2006/uk-2006-05-beta-1-1-10.std');
[sstdx2 pstdx2]=sort(temp,1,'descend');

n = length(sxeA1);

%% 
% Generate a reasonable set of points sampled from this space.  The results
% of this operation is sorted.  
pts = union(floor(logspace(0,log10(n),1000)), floor(linspace(1,n,1000)));
ms={'MarkerSize',15}; set(0,'defaultaxesfontsize',12); 
set(0,'defaultaxeslinewidth',0.8,'defaultlinelinewidth',1);

%%
% Plot the PageRank values
figure(1);h=loglog(pts,sxeA1(pts),'k-.',pts,sxeA2(pts),'k:');set(h(1),ms{:});
hold on;g=loglog(pts,sstdx1(pts),'k-',pts,sstdx2(pts),'k--');set(g(1),ms{:});
hold off; legend('x(E(A_1))', 'x(E(A_2))', 'Std[x(A_1)]','Std[x(A_2)]');

%%
% Compute the intersection metrics
[im1a] = intersectmetric(pxeA1,pstdx1); im1a=im1a(pts);
[im2a] = intersectmetric(pxeA2,pstdx2); im2a=im2a(pts);

%% 
% Plot and save
figure(2); semilogx(pts,im1a,'k-',pts,im2a,'k--'); pbaspect([4 2 1]);
p=get(gcf,'Position');p(3)=800;set(gcf,'Position',p);
xlim([1 n]); ylim([0 1]); ylabel('Intersection Similarity (k)'); xlabel('k');
text(-0.05,1, 'Disjoint','HorizontalAlignment','Right','Units','Normalized'); 
text(-0.05,0, 'Identical','HorizontalAlignment','Right','Units','Normalized');
legend('Std[x(A_1)] vs. x(0.85)', 'Std[x(A_2)] vs. x(0.5)'); legend boxoff;
print(gcf,'uk-2006-isim.eps','-depsc2','-cmyk');