%% Accurate RAPr computations with the Symbolic Toolbox
% Our initial attempt at using the symbolic toolbox may have been slightly
% incorrect.  This code attempts to use the symbolic 

%% Experimental setup
% This experiment should be run from the rapr/experiments/correctness
% directory.  To reduce the computational load, we'll load the cs-stanford
% graph and just take the largest strong component.
cwd = pwd;
dirtail = 'experiments/correctness'; 
if strcmp(cwd(end-length(dirtail)+1:end),dirtail) == 0
    warning('%s should be executed from rapr/%s\n', mfilename, dirtail);
end
load ('../../data/harvard500.mat');
% just use the largest strong component for this experiment;
P = Pcc; A = Acc;
%load ('../../data/example-small.mat'); A(1,:) = ones(1,size(A,1));
n=size(P,1);
v=ones(n,1)./n;

%%
% Add the path 
addpath('../../matlab');

%% Generate the linear system

%%
% 
clear alpha;
alpha=sym('alpha','real');

%%
%
e = sym(ones(n,1));
b = (1-alpha)*e/n;
fprintf('... build identity matrix ... '); tic
M = diag(e);
fprintf('... done! %f sec\n', toc);

%%
%
fprintf('... assembling Pagerank matrix ...\n'); tic
[nzi nzj nzv] = find(A);
d = sum(A,2); nextupdate = floor(nnz(A)/100);
for i=1:nnz(A)
    M(nzj(i),nzi(i))=M(nzj(i),nzi(i))-alpha/d(nzi(i));
    if i > nextupdate, 
        fprintf('... ... %i/%i in %7f sec\n', i, nnz(A),toc); 
        nextupdate=nextupdate+floor(nnz(A)/100);
    end
end

%%
% 
fprintf('... solving symbolic system ...\n'); tic
x = M\b;
fprintf('... done! %f sec\n', toc);

save 'exact-sym-inv.mat' x

%% Write the functions for Mathematica
syms a;
xa = subs(x,alpha,a);
fid=fopen('exact-sym-inv.mf','wt');
for i=1:n, fprintf(fid,'%s\n',char(xa(i))); end
fclose(fid);
clear a;


%% Integrate expressions exactly

% Setup our standard list of parameters
params = [
      0,   0, 0.6, 0.9   % uniform [0,6,0.9]
      2,  16,   0,   1   % skewed right
      1,   1, 0.1, 0.9   % equicentric
   -0.5,-0.5, 0.2, 0.7   % bi-modal 
];
for i=1:size(params,1)
    a=params(i,1);b=params(i,2); l=params(i,3); r=params(i,4);
    results(i).d = alphadist('beta',a,b,l,r);
    f = (alpha-sym(l))^b*(sym(r)-alpha)^a;
    %wf = int(f,alpha,l,r);
    wf = (r-l)^(a+b+1)*beta(a+1,b+1);
    % computed un-normalized ex and ex2
    ex = zeros(n,1); ex2=ex; dpts=unique(round(linspace(1,n,100))); dp=1; tic;
    for j=1:n
        ex(j) = double(int(x(j)*f,alpha,l,r));
        ex2(j) = double(int(x(j)*f,alpha,l,r));
        if j>=dpts(dp)
            dt=toc; etr=n*dt/j; 
            fprintf('dist=%2i node=%6i etr=%7f sec\n',i,j,etr);
        end
    end
    %ex = double(int(x*f,alpha,l,r));
    %ex2 = double(int(x.^2*f,alpha,l,r));
    %ex = 2^(a+b+1)*ex./((r-l)*beta(a+1,b+1));
    %ex2 = 2^(a+b+1)*ex2./((r-l)*beta(a+1,b+1));
    ex = ex./double(wf);
    ex2 = ex2./double(wf);
    stdx = sqrt(ex2-ex.^2);
    exactex(:,i) = ex;
    exactstdx(:,i) = stdx;
    results(i).ex.exact = ex;
    results(i).stdx.exact = stdx;
end

%% Load results from files
for i=1:size(params,1)
    a=params(i,1);b=params(i,2); l=params(i,3); r=params(i,4);
    results(i).d = alphadist('beta',a,b,l,r);
    mathematicadata = struct2cell(load(sprintf('har500-b%i.ex.mat',i)));
    results(i).ex.exact = mathematicadata{1};
    mathematicadata = struct2cell(load(sprintf('har500-b%i.ex2.mat',i)));
    results(i).ex2.exact = mathematicadata{1};
    results(i).stdx.exact = sqrt(results(i).ex2.exact-results(i).ex.exact^2);
end

   
%% Compare against algorithms

%%
%
gqNs=0:10:100;
for i=1:length(results)
    eex = results(i).ex.exact; estdx = results(i).stdx.exact; d = results(i).d;
    gqconv=zeros(length(gqNs),3);
    for Ni=1:length(gqNs)
        N=gqNs(Ni);
        [ex stdx] = gqrapr(P,N+1,d,'direct');
        gqconv(Ni,:) = [N,norm(ex-eex,1),norm(stdx-estdx,1)];
    end
    results(i).conv.gq = gqconv;
    figure(i); semilogy(gqconv(:,1),gqconv(:,2),'.-',gqconv(:,1),gqconv(:,3),'*--');
end
