function s=csum(x)
% CSUM Compensated summation algorithm for a vector x.
%
% s=csum(x) returns the sum of all the elements in a vector x.
% 
% This code was copied from Accuracy and Stability of Numerical Algorithms,
% Higham, p. 84-85.  It implement's Kahan's final correction term described
% on p. 85. ("s=s+e");
%
% This algorithm should be numerically stable for sums of up to 10^16
% numbers.
%
% See also CSUM
%
% Example:
%  x=rand(1e5,1); x=x./csum(x);
%  fprintf('ssum(x) = %18.16e\n', csum(x));
%  fprintf(' sum(x) = %18.16e\n', sum(x));
%  fprintf('csum(x) = %18.16e\n', ssum(x));


% David Gleich, 2008

s=0; e=0; y=0; temp=0;
for i=1:numel(x)
    temp=s; y=x(i)+e; s=temp+y; e=(temp-s)+y;
end
s=s+e;