%% Compute statistics for all our datasets

%% Experimental setup
% This experiment should be run from the rapr/experiments/datastats directory
cwd = pwd;
dirtail = 'experiments/datastats'; 
if strcmp(cwd(end-length(dirtail)+1:end),dirtail) == 0
    warning('%s should be executed from rapr/%s\n', mfilename, dirtail);
end

datadir = '../../data';
wg_scc = '../../wg++/wg_scc';
wg_cores = '../../wg++/wg_cores';

matlab_bgl_path = '~/dev/matlab/matlab_bgl';
libbvg_path = '../../libbvg';

matlab_wg_file = 'matlab_bg_data';

%%
% List the graph names in two sets.  The first is a set of mat files, 
% and the second is a set of .graph files in the bvgraph format.  The
% two wg_* strings give the path to the wg_scc and wg_cores programs
% to compute the statistics for really large graphs without bringing
% the graph into Matlab directly.
matdata = {'wb-cs.stanford','uk-2006-05.hostgraph','uk-2007-05.hostgraph','generank'};
bvgdata = {'uk-2006-05','uk-2007-05',...
           'cnr-2000','eu-2005','in-2004','nz2006','uk-2005','us2004scc'};           


% for graphs with more than largedata edges, use wg_* funcs
largedata = 70e6; 

%%
% This experiment requires the MatlabBGL library, and the most recent 3.0
% release with the core_numbers function.  An official 3.0 (non-beta) 
% will (hopefully) be done soon.  If not, this experiment will still work
% for everything but the matdata terms and you'll have to set largedata=0

addpath(matlab_bgl_path);
addpath(libbvg_path);

%% Compute for matdata

results = [];
ngraphs = length(matdata);
for gi=1:ngraphs
    g = matdata{gi};
    results(gi).name = g;
    load([datadir filesep g '.mat']);
    % this loads P for the transition probs.
    A = spones(P); % just make sure
    
    results(gi).nverts = size(A,1);
    results(gi).nedges = nnz(A);
    
    tic; 
    x = sum(A,1); % x = indegree
    results(gi).timeid = toc;;
    results(gi).maxid = max(x);
    results(gi).nzeroid = sum(x==0);
    tic; 
    x = sum(A,2); % x = outdegree
    results(gi).timeod = toc;
    results(gi).maxod = max(x);
    results(gi).nzerood = sum(x==0);
    
    tic; 
    x = components(A);
    results(gi).timecomps = toc;
    ccs = accumarray(x,1);
    results(gi).ncomp = max(x);
    results(gi).maxcompsize = max(ccs);
    results(gi).nisocomp = sum(ccs==1);
    clear ccs;
    
    tic;
    x = core_numbers(A);
    results(gi).timecores = toc;
    cns = accumarray(x+1,1);
    results(gi).maxcorenum = max(x);
    results(gi).zerocoresize = sum(x==0);
    results(gi).onecoresize = sum(x==1);
    [cs,cn] = max(cns);
    results(gi).maxcore = [cs cn-1];
    clear cns cs cn;
    
    results(gi), fprintf('\n');
end

matresults = results;
save 'matresults' matresults;

%% Compute for bvgdata

results = [];
ngraphs = length(bvgdata);
for gi=1:ngraphs
    g = bvgdata{gi}; graphbase = [datadir filesep g];
    results(gi).name = g;
    
    A = bvgraph(graphbase,struct('load_type','offline')); % don't load into mem
    
    results(gi).nverts = size(A,1);
    results(gi).nedges = nnz(A);
    
    tic; 
    x = sum(A,1); % x = indegree
    results(gi).timeid = toc;
    results(gi).maxid = max(x);
    results(gi).nzeroid = sum(x==0);
    tic; 
    x = sum(A,2); % x = outdegree
    results(gi).timeod = toc;
    results(gi).maxod = max(x);
    results(gi).nzerood = sum(x==0);
    
    % add nnz(A) < 0 to fix a bug with data larger than 2^31
    if nnz(A) > largedata || nnz(A) < 0
        % run with wg codes
        tic;
        system(sprintf('%s %s > %s',wg_scc,graphbase,matlab_wg_file));
        results(gi).timecomps = toc;
        x = load(matlab_wg_file);
        x = x+1;
        
        ccs = accumarray(x,1);
        results(gi).ncomp = max(x);
        results(gi).maxcompsize = max(ccs);
        results(gi).nisocomp = sum(ccs==1);
        clear x ccs;
        
        tic;
        system(sprintf('%s %s > %s',wg_cores,graphbase,matlab_wg_file));
        results(gi).timecores= toc;
        x = load(matlab_wg_file);
        
        cns = accumarray(x+1,1);
        results(gi).maxcorenum = max(x);
        results(gi).zerocoresize = sum(x==0);
        results(gi).onecoresize = sum(x==1);
        [cs,cn] = max(cns);
        results(gi).maxcore = [cs cn-1];
        clear cns cs cn;
        
    else
        % load as a Matlab graph
        G = A;
        A = sparse(G);
        
        tic; 
        x = components(A);
        results(gi).timecomps = toc;
        ccs = accumarray(x,1);
        results(gi).ncomp = max(x);
        results(gi).maxcompsize = max(ccs);
        results(gi).nisocomp = sum(ccs==1);
        clear ccs;
        
        tic;
        x = core_numbers(A);
        results(gi).timecores = toc;
        cns = accumarray(x+1,1);
        results(gi).maxcorenum = max(x);
        results(gi).zerocoresize = sum(x==0);
        results(gi).onecoresize = sum(x==1);
        [cs,cn] = max(cns);
        results(gi).maxcore = [cs cn-1];
        clear cns cs cn;
    end
    
    results(gi), fprintf('\n');
end

bvgresults = results;
save 'bvgresults' bvgresults;
