%% Are GQ and PCE different?
% Some initial tests showed surprisingly similar results from PCE and GQ.
% Let's test some ideas with that in this code.

%% Setup the experiment
addpath('../../matlab');
addpath('../../libbvg');
datadir = '../../data';

%% Load our small test graph
G = bvgraph([datadir filesep 'wb-cs.stanford']);
P = ssbvgraph(G);
Ns=2:10; exdiff = []; stddiff=[];
for Ni=1:length(Ns)
    N=Ns(Ni);
    [gqex gqstd] = gqrapr(P,N);
    [pceex pcestd] = pcerapr(P,N-1);
    exdiff(Ni) = norm(gqex-pceex,1);
    stddiff(Ni) = norm(gqstd-pcestd,1);
end

%% Plot the results
subplot(2,1,1);
plot(Ns,exdiff,'.-'); xlabel('N'); ylabel('Difference in expectation');
subplot(2,1,2);
plot(Ns,stddiff,'.-'); xlabel('N'); ylabel('Difference in stdev');

%% Load our small test graph 
% and use a direct algorithm this time
load([datadir filesep 'wb-cs.stanford.mat']);
P = Pcc;
Ns=2:10; exdiff = []; stddiff=[];
for Ni=1:length(Ns)
    N=Ns(Ni);
    [gqex gqstd] = gqrapr(P,N,[],'direct');
    [pceex pcestd] = pcerapr(P,N-1,[],'direct');
    exdiff(Ni) = norm(gqex-pceex,1);
    stddiff(Ni) = norm(gqstd-pcestd,1);
end

%% Plot the results
subplot(2,1,1);
plot(Ns,exdiff,'.-'); xlabel('N'); ylabel('Difference in expectation');
subplot(2,1,2);
plot(Ns,stddiff,'.-'); xlabel('N'); ylabel('Difference in stdev');
