function [tau, c, d, tr, ts, j, n] = ktau(v, varargin);
% KTAU Compute Kendall's tau and all associated parameters.  
%
% tau = ktau(V) 
% tau = ktau(v1, v2)
%
% Computes the Kendall's tau coefficient between two rankings.  Either v1
% and v2 or V(:,1), V(:,2)
%
% v1, v2 

%
% David Gleich
% Stanford University
% 8 February 2006
%

if (nargin == 1)
    v1 = v(:,1);
    v2 = v(:,2);
else
    v1 = v;
    v2 = varargin{1};
end;

try
    res = it.unimi.dsi.law.stat.KendallTau.computeStats(v1, v2, []);
catch
    mypath = which('ktau');
    [pathstr,filename,ext,VERSN] = fileparts(mypath);
    javaaddpath([pathstr filesep() 'ktau_lib' filesep() 'colt.jar']);
    javaaddpath([pathstr filesep() 'ktau_lib' filesep() 'fastutil-4.4.0.jar']);
    javaaddpath([pathstr filesep() 'ktau_lib' filesep() 'mg4j-0.9.1.jar']);
    javaaddpath([pathstr filesep() 'ktau_lib' filesep() 'law-1.0.2' filesep()]);
    
    res = it.unimi.dsi.law.stat.KendallTau.computeStats(v1, v2, []);
end;

d = double(res(3));
tr = double(res(1));
ts = double(res(2));
j = double(res(4));

n = length(v1)*(length(v1)-1)/2;

c = n - (tr + ts - j) - d;

tau = (n - (tr + ts - j) - 2*d)/(sqrt(n - tr)*sqrt(n - ts));
