
%% Setup the computation
% check to make sure the working directory is correct...
cwd = pwd;
dirtail = 'libbvg'; 
if strcmp(cwd(end-length(dirtail)+1:end),dirtail) == 0
    warning('%s should be executed from rapr/%s\n', mfilename, dirtail);
end
datadir='../data';

%% Test mcrapr with inout algorithm
fprintf('... testing mcrapr with inout ...\n');
% run mcrapr
!./gqrapr ../data/wb-cs.stanford _ testrun inout

% load the graph and define a pagerank function in terms of alpha
G = bvgraph([datadir filesep 'wb-cs.stanford']);
A = sparse(G);
d = sum(A,2);
[i j v] = find(A);
P = sparse(i,j,v./d(i),size(A,1),size(A,2));
n = size(P,1);
pr1 = @(a) (speye(n)-a*P')\ones(n,1)./n;
norm1 = @(x) x./norm(x,1);
pr = @(a) norm1(pr1(a));

% compute everything in Matlab
xw = load('testrun.xw'); 
alphas=xw(:,1);
Xs = zeros(size(P,1),length(alphas));
for i=1:length(alphas), Xs(:,i) = pr(alphas(i)); end

% load the answers
m2 = binload('testrun.mean');
std2 = binload('testrun.std');

% compute the difference
fprintf('1-norm diffence in mean for c/matlab: %g\n', norm(m2 - Xs*xw(:,2),1));
stdM = sqrt(Xs.^2*xw(:,2) - (Xs*xw(:,2)).^2);
fprintf('1-norm diffence in  std for c/matlab: %g\n', norm(std2 - stdM,1));

%% Test mcrapr with power method
fprintf('... testing mcrapr with power ...\n');
% run mcrapr
!./gqrapr ../data/wb-cs.stanford _ testrun power

% load the graph and define a pagerank function in terms of alpha
G = bvgraph([datadir filesep 'wb-cs.stanford']);
A = sparse(G);
d = sum(A,2);
[i j v] = find(A);
P = sparse(i,j,v./d(i),size(A,1),size(A,2));
n = size(P,1);
pr1 = @(a) (speye(n)-a*P')\ones(n,1)./n;
norm1 = @(x) x./norm(x,1);
pr = @(a) norm1(pr1(a));

% compute everything in Matlab
xw = load('testrun.xw');
Xs = zeros(size(P,1),length(alphas));
for i=1:length(alphas), Xs(:,i) = pr(alphas(i)); end

% load the answers
m2 = binload('testrun.mean');
std2 = binload('testrun.std');

% compute the difference
fprintf('1-norm diffence in mean for c/matlab: %g\n', norm(m2 - Xs*xw(:,2),1));
stdM = sqrt(Xs.^2*xw(:,2) - (Xs*xw(:,2)).^2);
fprintf('1-norm diffence in  std for c/matlab: %g\n', norm(std2 - stdM,1));

