function m=beta_moments(N,a,b,l,r)
% BETA_MOMENTS Compute moments of the four parameter beta distribution (a,b,l,r)
%
% m=beta_moments(N,a,b) computes the 0,...,N moments of the Beta(a,b)
% distribution on [0,1].
% m=beta_moments(N,a,b,l,r) computes the 0,...,N moments of the Beta(a,b)
% distribution on [l,r].
%
% The parameters of the beta distribution used in this function are 
% different than the one implemented inside of Matlab.  For example,
%   Beta(a,b) = MatlabBeta(b+1,a+1).
% and Beta(0,0) is a uniform distribution, not Beta(1,1).
%
% See also UNIF_MOMENTS
%
% Example:
%   m=beta_moments(10,1,2);             % m(2) = E[Beta(1,2)] = 2/3
%   mbu=beta_moments(10,0,0,0.4,0.6);   % mbu(2) = E[Unif(0.4,0.6)] = 1/2
%   mu=unif_moments(10,0.4,0.6);        % mu = mbu

% David Gleich and Paul Constantine
% Copyright, Stanford University, 2008

% 2008 February 1
% Initial version
%
% 11 February 2008
% Removed p power

if ~exist('l','var') || isempty(l), l=0; end
if ~exist('r','var') || isempty(r), r=1; end

% initialize
c = l; s = (r-l); m=zeros(N+1,1);  % c is the offset, s is the scale
uk=1; k=0; sk=1; m(1) = uk; % note that r is 
for i=1:N, k = k+1; uk=s*uk*((b+k)/(a+b+k+1)); m(i+1) = uk; end 
% form the shifted moments
if c ~= 0, for i=1:N, m(i+1:end) = c*m(i:(end-1)) + m((i+1):end); end, end
