function xw=beta_qrule(N,a,b,l,r)
% BETA_QRULE Generate the Gauss Quadrature rule for a beta distribution
%
% xw = beta_qrule(N,a,b) computes the N point Gauss Quadrature rule for a
% beta distribution on [a,b] via the quadrature points for a Jacobi weight
% function.
%
% The quadrature points are xw(:,1) and the quadrature weights are xw(:,2).
% 
% This code uses the OQP codes from Walter Gautschi.
%
% Example:
%   beta_qrule(3,2,16,0,1)
%   beta_qrule(3,0,0,0,1)

% Paul Constantine and David Gleich
% Copyright, Stanford University, 2008

% 2008 February 1
% Initial version

ab=r_jacobi01(N,a,b);
xw=gauss(N,ab);
xw(:,2) = (1./beta(b+1,a+1))*xw(:,2);
xw(:,1) = (r-l).*xw(:,1)+l;