function Xs = solve_pcesys_diag(P,T,u,v,tol,maxit)
% SOLVE_PCESYS_DIAG Solve a PCE linear system by diagonalization.
%
% Xs = solve_pcesys_diag(P,T,u,v,tol,maxit) solves
% (kron(eye(N+1),speye(n)) - kron(T,P'+vd'))*reshape(Xs,(N+1)*n,1) = kron(u,v)
% and ignores tol and maxit.
%
% To do this, it diagonalizes the matrix T, which induces a diagonalization
% of the matrix (kron(eye(N+1),speye(n)) - kron(T,P'+vd')).  Consequently,
% we get a series of rescaled PageRank systems that we can solve.

N = size(T,1); n = size(P,1);
[Z L] = eig(T); 
q=Z\u; Y=zeros(n,N);
for i=1:length(q)
    Y(:,i) = inoutpr(P,L(i,i),v,tol,maxit);
    Y(:,i) = (q(i)/(1-L(i,i)))*Y(:,i); 
end
Xs=Y*Z';