function Xs = solve_pcesys_direct(P,T,u,v,tol,maxit)
% SOLVE_PCESYS_DIRECT Solve a PCE linear system with Matlab's backslash
%
% Xs = solve_pcesys_direct(P,T,u,v,tol,maxit) solves
% (kron(eye(N+1),speye(n)) - kron(T,P'+vd'))*reshape(Xs,(N+1)*n,1) = kron(u,v)
% and ignores tol and maxit.


N = size(T,1)-1; n = size(P,1);
d = P*ones(n,1) < 0.5;
v = zeros(n,1) + v;
if nnz(d) > 0, P = P+sparse(d)*v'; end
A = kron(speye(N+1),speye(n)) - kron(sparse(T),P');
Xs = A\kron(u,v);
Xs = reshape(Xs,n,N+1);
