function Xs = solve_pcesys_jacobi(P,T,u,v,tol,maxit)
% SOLVE_PCESYS_JACOBI Solve a PCE linear system with a Jacobi method
%
% Xs = solve_pcesys_jacobi(P,T,u,v,tol,maxit) solves
% (kron(eye(N+1),speye(n)) - kron(T,P'+vd'))*reshape(Xs,(N+1)*n,1) = kron(u,v)
% and so that ||b-Ax||_1 <= tol or we reach maxit.

N = size(T,1)-1; n = size(P,1);
T = sparse(T); u = sparse(u); Tt = T';
Xs = zeros(n,N+1); Ys = zeros(n,N+1);

delta = 2; iter = 0;
while iter < maxit && delta > tol
    xsum = sum(Xs);
    Ys(:) = 0;
    for i=1:N+1, 
        for j=find(Tt(:,i))'
            y = P'*Xs(:,j); y = y+(xsum(j)-sum(y)).*v;
            Ys(:,i) = Ys(:,i) + T(i,j)*y;
        end
        if u(i) ~= 0, Ys(:,i) = Ys(:,i) + u(i).*v; end
    end
    Xs = Xs - Ys; Xs = abs(Xs); delta = sum(Xs(:));
    fprintf('jacobi : iter = %6i ; delta = %10e\n', iter, delta);
    Xs = Ys; iter = iter + 1;
end
