function Xs = solve_pcesys_ujacobi(P,T,u,v,tol,maxit)
% SOLVE_PCESYS_JACOBI Solve a PCE linear system with a Jacobi method
%
% Xs = solve_pcesys_jacobi(P,T,u,v,tol,maxit) solves
% (kron(eye(N+1),speye(n)) - kron(T,P'+vd'))*reshape(Xs,(N+1)*n,1) = kron(u,v)
% and so that ||b-Ax||_1 <= tol or we reach maxit.

N = size(T,1)-1; n = size(P,1);
T = sparse(T); u = sparse(u); Tt = T';
Xs = zeros(n,N+1); xi = zeros(n,1);

delta = 2; iter = 0;
while iter < maxit && delta > tol
    delta = 0; 
    for i=1:N+1, 
        xi(:) = 0;
        for j=find(Tt(:,i))'
            y = P'*Xs(:,j); y = y+(sum(Xs(:,j))-sum(y)).*v;
            xi = xi + T(i,j)*y;
        end
        if u(i) ~= 0, xi = xi + u(i).*v; end
        Xs(:,i) = Xs(:,i) - xi; delta = delta + norm(Xs(:,i),1);
        Xs(:,i) = xi;
    end
    fprintf('ujacobi : iter = %6i ; delta = %10e\n', iter, delta);
    iter = iter + 1;
end
