function xw=beta_qrule2(N,a,b,l,r)
% BETA_QRULE2 Generate the Gauss Quadrature rule for a beta distribution
%
% xw = beta_qrule(N,a,b) computes the N point Gauss Quadrature rule for a
% beta distribution on [a,b] via the quadrature points for a Jacobi weight
% function over [-1,1].  
%
% The quadrature points are xw(:,1) and the quadrature weights are xw(:,2).
%
% This function should produce identical output to beta_qrule and is used
% for testing.
% 
% This code uses the OQP codes from Walter Gautschi.

ab=r_jacobi(N,a,b);
xw=gauss(N,ab);
xw(:,2) = xw(:,2)./(beta(b+1,a+1)*2^(a+b+1));
xw(:,1) = (r-l).*xw(:,1)./2 + (r+l)./2;