function test_main
%% Setup tests
msgid = 'rapr:testFailed'; % define the error message id
cwd = pwd;
dirtail = 'matlab'; 
if strcmp(cwd(end-length(dirtail)+1:end),dirtail) == 0
    warning('%s should be executed from rapr/%s\n', mfilename, dirtail);
end


%% Test sample code

%%
% alphadist
d=alphadist('beta'); hist(d.rnd(1000),100); % plot a histogram
d=alphadist('unif',0.3,0.6); hist(d.rnd(1000),100); % plot a histogram
d=alphadist('beta',2,16,0,0.95); hist(d.rnd(1000),100); % plot a histogram

%%
% unif_moments
m=unif_moments(10,0.4,0.6);      % Moments of unif(0.4,0.6)
mb=beta_moments(10,0,0,0.4,0.6); % Moments of unif(0.4,0.6) from Beta
if norm(m-mb,1)>1e-15, error(msgid, 'differences in beta and uniform'); end

%%
% beta_moments
m=beta_moments(10,1,2);             % m(2) = E[Beta(1,2)] = 2/3
mbu=beta_moments(10,0,0,0.4,0.6);   % mbu(2) = E[Unif(0.4,0.6)] = 1/2
mu=unif_moments(10,0.4,0.6);        % mu = mbu
if norm(mu-mbu,1)>1e-15, error(msgid, 'differences in beta and uniform'); end

%%
% unif_qrule
xwu=unif_qrule(5,0,1);
xwb=beta_qrule(5,0,0,0,1);
if norm(xwu-xwb,1) > 100*eps,
    error(msgid,'unif_qrule example failed');
end

%%
% beta_qrule
beta_qrule(3,2,16,0,1);
beta_qrule(3,0,0,0,1);

%%
% beta_normorthprods
[h2 T u] = beta_normorthprods(4,1,1,0,1);
[h2 T u] = beta_normorthprods(4,0,0,0,1); % get products for uniform

%%
% beta_orthprods
[h2 T u] = beta_orthprods(4,1,1,0,1);
[h2 nT u] = beta_normorthprods(4,1,1,0,1);
if norm(T - diag(h2)*nT,1) > 100*eps(1)
     error(msgid,'beta_orthprods example failed');
end

%% 
% powerpr
load('../data/wb-cs.stanford.mat');
x=powerpr(P);

if abs(norm(x,1)-1)>1e-15, error(msgid, 'pagerank vector not normalized'); end

%% 
% mcrapr
load('../data/example-small.mat');
tic, [ex stdx] = mcrapr(P,10000,[],'direct'); toc
tic, [ex stdx] = mcrapr(P,10000); toc      % sometimes exact is faster!

%%
% pdrapr
load('../data/example-small.mat');
[ex stdx] = pdrapr(P);
[ex stdx] = pdrapr(P,[],-1e-6); % actually compute stdx

%%
% gqrapr
load('../data/example-small.mat');
[ex stdx] = gqrapr(P,10);
[ex stdx] = gqrapr(P,20,alphadist('beta',2,16),'direct');

%%
% pcerapr
load('../data/example-small.mat');
[ex stdx] = pcerapr(P,4);
[ex stdx] = pcerapr(P,9,alphadist('beta',2,16),'direct');

%% Test distributions

%% 
% Test the uniform distribution

%%
% Test the beta distribution


N=4; ab = [1 1]; lr=[0.1, 0.9];
d = alphadist('beta',ab(1),ab(2),lr(1),lr(2));
u=d.mf(4);
if norm(u-[1 1/2 141/500 173/1000 39443/350000]',1) > 100*eps
    error(msgid,'beta_moments(%i,%g,%g,%g,%g) not correct',...
        N,ab(1),ab(2),lr(1),lr(2));
end

N=4; ab = [4 3]; lr=[2/7, 9/10];
d = alphadist('beta',ab(1),ab(2),lr(1),lr(2));
u=d.mf(4);
if norm(u-[1 176/315 2363/7350 76871/404250 39181361/339570000]',1) > 100*eps
    error(msgid,'beta_moments(%i,%g,%g,%g,%g) not correct',...
        N,ab(1),ab(2),lr(1),lr(2));
end

%
% test the moments of the beta
%
for i=1:100
    ab=floor(16*rand(2,1));
    d = alphadist('beta',ab(1),ab(2),0,1);
    N = 500;
    u = d.mf(N);
    u2 = [1 cumprod((ab(2)+1+(0:N-1))./(ab(1)+ab(2)+2+(0:N-1)))];
    if norm(u-u2',1) > 100*eps,
        error(msgid,'beta_moments(%i,%g,%g,%g,%g) not correct',...
            N,ab(1),ab(2),0,1);
    end
end

%
% test the moments of the uniform with shifts against the beta with shifts
%
for i=1:100
	lr=sort(rand(2,1));
    d = alphadist('beta',0,0,lr(1),lr(2));
    d2 = alphadist('unif',lr(1),lr(2));
    N = 500;
    u = d.mf(N);
    u2 = d2.mf(N);
    if norm(u-u2,1) > 100*eps,
        error(msgid,'unif_moments(%i,%g,%g) not correct',...
            N,lr(1),lr(2));
    end
end    

% test a set of quadrature rules against the qrule_beta2 function
rand('state',0); % reset random generator
for i=1:100
    ab=floor(16*rand(2,1)); lr=sort(rand(2,1));
    d = alphadist('beta',ab(1),ab(2),lr(1),lr(2));
    N=2*floor(16*rand(1))+1; xw = d.qrule(N); 
    xw = d.qrule(N); xw2 = beta_qrule2(N,ab(1),ab(2),lr(1),lr(2));
    if any(abs(xw - xw2) > 100*eps),
        error(msgid,'beta_qrule(%i,%g,%g,%g,%g) not correct',...
            N,ab(1),ab(2),lr(1),lr(2));
    end
    if norm(xw(:,2),1) < 100*eps, 
        error(msgid,'beta_qrule(%i,%g,%g,%g,%g) norm incorrect',...
            N,ab(1),ab(2),lr(1),lr(2)); 
    end
end

% test the midpoint of the quadrature rule, which is the midpoint of the 
% [l,r] interval if a=b.
for i=1:100
    ab=floor(16*rand(2,1)); lr=sort(rand(2,1));
    d=alphadist('beta',ab(1),ab(1),lr(1),lr(2));
    N=2*floor(16*rand(1))+1; xw = d.qrule(N); 
    midpt = xw(floor(size(xw,1)/2)+1,1);
    ex = (lr(2)-lr(1))/2+lr(1);
    if abs(midpt - ex) > 100*eps,
        error(msgid,'beta_qrule misses midpoint for beta_qrule(%i,%g,%g,%g,%g)',...
            N,ab(1),ab(2),lr(1),lr(2));
    end
    if norm(xw(:,2),1) < 100*eps, 
        error(msgid,'beta_qrule(%i,%g,%g,%g,%g) norm incorrect',...
            N,ab(1),ab(2),lr(1),lr(2)); 
    end
end

%
% Test the pce coefficients of the beta distribution
%
ab = [2,16]; lr=[0.1,0.9]; N=4;
d=alphadist('beta',ab(1),ab(2),lr(1),lr(2));
[h2 T u] = d.normorthprods(4);
Tt = ...
[39/50 17/175 0 0 0
 1/25  401/550 144/1265 0 0
 0 16/231 38/55 19/150 0
 0 0 23/230 43/65 16/117
 0 0 0 176/1625 83/130];
ut = [11/50 -1/25 0 0 0];
h2t = [274877906944/55446624755859375
    274877906944/22830963134765625
    824633720832/41675567626953125
    274877906944/10013580322265625
    274877906944/7930755615234375];

if any(abs(Tt-T)>10*eps),
    error(msgid,'beta_normorthprods(%i,%g,%g,%g,%g) incorrect',...
            N,ab(1),ab(2),lr(1),lr(2)); 
end
if any(abs(u-ut')>10*eps),
    error(msgid,'beta_normorthprods(%i,%g,%g,%g,%g) incorrect',...
            N,ab(1),ab(2),lr(1),lr(2)); 
end
if any(abs(h2-h2t)>10*eps),
    error(msgid,'beta_normorthprods(%i,%g,%g,%g,%g) incorrect',...
            N,ab(1),ab(2),lr(1),lr(2)); 
end


for i=1:100
    ab=floor(16*rand(2,1)); lr=sort(rand(2,1));
    d=alphadist('beta',ab(1),ab(2),lr(1),lr(2));
    N=2*ceil(16*rand(1))+1; xw = d.qrule(N); 
    [h2 T u] = d.normorthprods(N-1);
    evals = eig(T);
    if any(sort(evals) - sort(xw(:,1)) > 100*eps),
        error(msgid,'beta_normorthprods(%i,%g,%g,%g,%g) wrong eigenvalues',...
            N,ab(1),ab(2),lr(1),lr(2)); 
    end
end

%
% Test the pce coefficients of the uniform distribution
% against those of the beta distribution
% 
for i=1:100
    ab=floor(16*rand(2,1)); lr=sort(rand(2,1));
    d=alphadist('unif',lr(1),lr(2));
    N=2*ceil(16*rand(1))+1; xw = d.qrule(N); 
    [h2 T u] = d.normorthprods(N-1);
    evals = eig(T);
    if any(sort(evals) - sort(xw(:,1)) > 100*eps),
        error(msgid,'beta_normorthprods(%i,%g,%g,%g,%g) wrong eigenvalues',...
            N,ab(1),ab(2),lr(1),lr(2)); 
    end
end

%
% Test the unnormalized PCE coefficients against the normalized ones.
%

for i=1:100
    ab=floor(16*rand(2,1)); lr=sort(rand(2,1));
    d=alphadist('beta',ab(1),ab(2),lr(1),lr(2));
    N=2*ceil(16*rand(1)); xw = d.qrule(N); 
    [nh2 nT u] = d.normorthprods(N-1);
    [h2 T] = d.orthprods(N-1);
    T2 = diag(h2)*nT;
    
    if any(abs(nh2-h2) > 100*eps),
        error(msgid,'beta_normorthprods(%i,%g,%g,%g,%g) wrong norms',...
            N,ab(1),ab(2),lr(1),lr(2)); 
    end
    if any(abs(T2 - T) > 100*eps),
        error(msgid,'beta_normorthprods(%i,%g,%g,%g,%g) wrong values',...
            N,ab(1),ab(2),lr(1),lr(2)); 
    end
end
    
%% 
% Test powerpr
addpath('../libbvg'); % add bvgraph package
load('../data/example-small.mat');
[x flag]=powerpr(P,[],[],[],5); % small number of iterations
if ~flag, error(msgid,'powerpr did not detect non-convegence'); end
load('../data/example-tiny.mat');
for i=1:100
    a=rand(1,1)*0.95;
    x = powerpr(P,a);
    if any(abs(x-[(1-a)/3; (2-a-a^2)/6; (2+3*a+a^2)/6])>1e-11),
        error(msgid,'powerpr generated incorrect vector');
    end
end

% test equivalence on ssbvgraph
G = bvgraph('../data/cnr-2000');
P = ssbvgraph(G);
x = powerpr(P);
A = sparse(G);
P = normout(A);
x2 = powerpr(P);
if norm(x-x2,1)>1e-11,
    error(msgid, 'powerpr failed bvgraph equivalence test');
end

% test accuracy of powerpr on tinygraph
G = bvgraph('../data/example-tiny');
P = ssbvgraph(G);
for i=1:100
    a=rand(1,1)*0.95;
    x = powerpr(P,a);
    if any(abs(x-[(1-a)/3; (2-a-a^2)/6; (2+3*a+a^2)/6])>1e-11),
        error(msgid,'powerpr generated incorrect vector');
    end
end

%% 
% Test inoutpr
addpath('../libbvg'); % add bvgraph package
load('../data/example-small.mat');
[x flag]=powerpr(P,[],[],[],5); % small number of iterations
if ~flag, error(msgid,'powerpr did not detect non-convegence'); end
load('../data/example-tiny.mat');
for i=1:100
    a=rand(1,1)*0.95;
    x = inoutpr(P,a);
    if any(abs(x-[(1-a)/3; (2-a-a^2)/6; (2+3*a+a^2)/6])>1e-11),
        error(msgid,'powerpr generated incorrect vector');
    end
end

% test equivalence on ssbvgraph
G = bvgraph('../data/cnr-2000');
P = ssbvgraph(G);
x = inoutpr(P);
A = sparse(G);
P = normout(A);
x2 = inoutpr(P);
if norm(x-x2,1)>1e-11,
    error(msgid, 'powerpr failed bvgraph equivalence test');
end

% test accuracy of powerpr on tinygraph
G = bvgraph('../data/example-tiny');
P = ssbvgraph(G);
for i=1:100
    a=rand(1,1)*0.95;
    x = inoutpr(P,a);
    if any(abs(x-[(1-a)/3; (2-a-a^2)/6; (2+3*a+a^2)/6])>1e-11),
        error(msgid,'powerpr generated incorrect vector');
    end
end

%%
% test_inoutpr vs. powerpr

test_inoutpr

%%
    

%% Test computations

%%
% pdrapr
[ex stdx] = pdrapr(P,[],-1e-6,150);

end