/*               
 * Portions copyright (c) 2003-2007, Paolo Boldi and Sebastiano Vigna. Translation copyright (c) 2007, Jacob Ratkiewicz
 *
 *  This program is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by the Free
 *  Software Foundation; either version 2 of the License, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 *  for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */

#ifndef DEBUG_IBITSTREAM_HPP
#define DEBUG_IBITSTREAM_HPP

#include <iostream>
#include <vector>
#include <fstream>
#include <cassert>

#include <boost/shared_ptr.hpp>

#include "../input_bitstream.hpp"

namespace webgraph {

/**
 * Class that logs all accesses to a bitstream, so that once can track them..
 */

class debug_ibitstream {

private:
   ibitstream& back;
   std::ofstream& log;

public:	
   typedef unsigned char byte;

   debug_ibitstream( ibitstream& b, std::ofstream& l ) : back(b), log(l) {}

   virtual ~debug_ibitstream() {}

   virtual void attach( boost::shared_ptr<std::vector<unsigned char> > buf ) {
      back.attach( buf );
   }

   virtual void flush() {
      back.flush();
   }
   
   long get_read_bits() {
      return back.get_read_bits();
   }
   
   void set_read_bits( long read_bits ) {
      back.set_read_bits( read_bits );
   }
   
   void read( byte bits[], unsigned int len ) {
      back.read( bits, len );
   }
   
   int read_bit() {
      return back.read_bit();
   }
   
   virtual int read_int( unsigned int len ) {
      
      int retval = back.read_int( len );
      log << "read_int( len ) = " << retval << "\n";

      return retval;
   }
   
   long skip( unsigned long n ) {
      return back.skip( n );
   }
   
   virtual void set_position( unsigned long position ) {
      back.set_position( position );
   }

   virtual int read_unary() {
      int retval = back.read_unary();
      log << "read_unary() = " << retval << "\n";
      return retval;
   }

   virtual int read_gamma() {
      int retval = back.read_gamma();
      log << "read_gamma() = " << retval << "\n";
      return retval;
   }

   virtual int read_delta() {
      int retval = back.read_delta();
      log << "read_delta() = " << retval << "\n";
      return retval;
   }

   virtual int read_zeta(int k) {
      int retval = back.read_zeta(k);
      log << "read_zeta() = " << retval << "\n";
      return retval;
   }

   virtual int read_nibble() {
      int retval = back.read_nibble();
      log << "read_nibble() = " << retval << "\n";
      return retval;
   }
};
} // end namespace?

#endif /*INPUT_BITSTREAM_HPP_*/
