/*               
 * Copyright (C) 2006 Jacob Ratkiewicz
 *
 *  This program is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by the Free
 *  Software Foundation; either version 2 of the License, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 *  for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */

/*
 * usage: given a compressed graph g (consisting of g.graph, g.offsets, and
 * g.properties), the following will run this program:
 * 
 *  wg_scc g
 *
 */
 
/*
 * 8 February 2008
 * Modified by David Gleich to compute the strong connected components of a 
 * a graph using boost's strong_components functions.
 */ 

#include <webgraph/boost/integration.hpp>
#include <iostream>
#include <boost/shared_ptr.hpp>
#include <cassert>
#include "core_numbers.hpp"
#include <boost/property_map.hpp>

int main(int argc, char** argv) {
    
    using namespace boost; 
    
   typedef webgraph::bv_graph::graph graph;
   typedef boost::shared_ptr<graph> graph_ptr;

   if (argc != 2) { 
    std::cerr << "usage: wg_scc graph" << endl;
    return (-1);
   }

   graph_ptr gp = webgraph::bv_graph::graph::load( argv[1] );
   
   cerr << "num vertices = " << num_vertices(*gp) << endl;
   cerr << "num edges = " << num_edges(*gp) << endl;

   typedef boost::graph_traits< graph > bv_traits;
   
   {
        vector<int> kcore_map(num_vertices(*gp));
        vector<int> pos_vec(num_vertices(*gp));
        {
            // compute core numbers components
            core_numbers(*gp, 
                make_iterator_property_map(kcore_map.begin(), identity_property_map()),
                make_core_numbers_visitor(null_visitor()),
                identity_property_map()
                //make_iterator_property_map(pos_vec.begin(), get(vertex_index, g))
            );
        };
        // write the output
        std::copy(kcore_map.begin(), kcore_map.end(), 
            ostream_iterator<int>(cout, "\n"));
   }

   /*cerr << "Here are the vertices ...." << endl;
   
   bv_traits::vertex_iterator v, v_end;
   for( tie( v, v_end ) = boost::vertices( *gp ); v != v_end; ++v ) {
      cerr << *v << endl;
   }
   
   cerr << "Here are the vertices again ...." << endl;
   
   bv_traits::vertex_iterator v2, v2_end;
   for( tie( v2, v2_end ) = boost::vertices( *gp ); v2 != v2_end; ++v2 ) {
      cerr << *v2 << endl;
   }

   cerr << "Now here are the edges..." << endl;
   
   bv_traits::edge_iterator e, e_end;
   for( tie( e, e_end ) = boost::edges( *gp ); e != e_end; ++e ) {
      cerr << "<" << e->first << ", " << e->second << ">" << endl;
   }*/

   return 0;
}
