/*               
 * Copyright (C) 2006 Jacob Ratkiewicz
 *
 *  This program is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by the Free
 *  Software Foundation; either version 2 of the License, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 *  for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */

/*
 * usage: given a compressed graph g (consisting of g.graph, g.offsets, and
 * g.properties), the following will run this program:
 * 
 *  wg_scc g
 *
 */
 
/*
 * 8 February 2008
 * Modified by David Gleich to compute the strong connected components of a 
 * a graph using boost's strong_components functions.
 */ 

#include <webgraph/boost/integration.hpp>
#include <iostream>
#include <boost/shared_ptr.hpp>
#include <cassert>
#include <boost/graph/strong_components.hpp>
#include <boost/property_map.hpp>

int main(int argc, char** argv) {
    
    using namespace boost; 
    
   typedef webgraph::bv_graph::graph graph;
   typedef boost::shared_ptr<graph> graph_ptr;

   if (argc != 2) { 
    std::cerr << "usage: wg_scc graph" << endl;
    return (-1);
   }

   graph_ptr gp = webgraph::bv_graph::graph::load( argv[1] );
   
   cerr << "num vertices = " << num_vertices(*gp) << endl;
   cerr << "num edges = " << num_edges(*gp) << endl;

   typedef boost::graph_traits< graph > bv_traits;
   
   {
        int num_components = 0;
        vector<int> component_map(num_vertices(*gp));
        {
            // compute connected components
            num_components = strong_components(*gp, 
                make_iterator_property_map(component_map.begin(),
                    identity_property_map()),
                    vertex_index_map(identity_property_map()));
        }

        vector<int> component_size(num_components);
        for (int i = 0; i < (int)component_map.size(); i++)
        {
            ++component_size[component_map[i]];
        }
        
        std::sort(component_size.begin(), component_size.end());

        cerr << "num components = " << num_components << endl;
        cerr << "largest component size(s) = ";
        for (int i = 0; i < std::min(5, num_components); i++)
        {
            cerr << component_size[num_components-1-i] << " ";
        }
        cerr << endl;

        std::sort(component_size.begin(), component_size.end());

        std::copy(component_map.begin(), component_map.end(), 
            ostream_iterator<int>(cout, "\n"));
   }

   /*cerr << "Here are the vertices ...." << endl;
   
   bv_traits::vertex_iterator v, v_end;
   for( tie( v, v_end ) = boost::vertices( *gp ); v != v_end; ++v ) {
      cerr << *v << endl;
   }
   
   cerr << "Here are the vertices again ...." << endl;
   
   bv_traits::vertex_iterator v2, v2_end;
   for( tie( v2, v2_end ) = boost::vertices( *gp ); v2 != v2_end; ++v2 ) {
      cerr << *v2 << endl;
   }

   cerr << "Now here are the edges..." << endl;
   
   bv_traits::edge_iterator e, e_end;
   for( tie( e, e_end ) = boost::edges( *gp ); e != e_end; ++e ) {
      cerr << "<" << e->first << ", " << e->second << ">" << endl;
   }*/

   return 0;
}
